#############################################################################
## 
## PolyLen( poly )
## 
## Computes the number of monomials in poly.
## 
## Input: poly = polynomial
## 
PolyLen := function( poly ) 
    return Length(ExtRepPolynomialRatFun(poly))/2; 
end;


#############################################################################
## 
## PolyDecomp( poly )
## 
## Computes the number of monomials and the total degree of poly.
## 
## Input: poly = polynomial
## 
PolyDecomp := function( poly )
   local indA, indB, indx, ext, mon, deg, i, e, m, d, j, k;

   indA := List(A, x -> ExtRepPolynomialRatFun(x)[1][1]);
   indB := List(B, x -> ExtRepPolynomialRatFun(x)[1][1]);
   indx := ExtRepPolynomialRatFun(x)[1][1];

   ext := ExtRepPolynomialRatFun(poly);
   mon := [];
   deg := 0;
   for i in [1,3..Length(ext)-1] do
       e := ext[i];
       m := 1;
       d := 0;
       for j in [1,3..Length(e)-1] do
           k := Position(indA, e[j]);
           if not IsBool(k) then 
               m := m*A[k]^e[j+1]; 
               d := d + e[j+1];
           fi;
           k := Position(indB, e[j]);
           if not IsBool(k) then 
               m := m*B[k]^e[j+1]; 
               d := d + e[j+1];
           fi;
           if e[j] = indx then 
               m := m*x^e[j+1]; 
               d := d + e[j+1];
           fi;
       od;
       AddSet(mon, m);
       deg := Maximum(deg, d);
   od;
   return rec( degree := deg, length := Length(mon));
end;


#############################################################################
## 
## PolyDeg( poly )
## 
## Computes the total degree of the polynomial poly.
## 
## Input: poly = polynomial
## 
PolyDeg := function( poly )
    local e, m, i, f;
    e := ExtRepPolynomialRatFun(poly);
    m := 0;
    for i in [1,3..Length(e)-1] do
        f := e[i];
        f := Sum(f{[2,4..Length(f)]}); 
        m := Maximum(m,f);
    od;
    return m;
end;


#############################################################################
## 
## GroupByVector( n, t )
## 
## Generate the group G( t ) of Hirsch-length n as group.
## 
## Input: n = Hirsch-length; t = parameters
## 
GroupByVector := function( n, t )
    local c, i, j, w, k;

    c := FromTheLeftCollector( n );
    for i in [1..n-2] do
        for j in [i+1..n-1] do
            w := [j, 1];
            for k in [j+1..Length( t[i][j] )] do
                Append( w, [k, t[i][j][k]] );
            od;
            SetConjugate( c, j, i, w );
        od;
    od;
    
    return PcpGroupByCollector( c );
end;


#############################################################################
## 
## VectorByGroupPcp( G )
## 
## Compute parameters t with G \cong G( t ), if possible. Uses the 
## given presentation.
## 
## Input: G = group
## 
VectorByGroupPcp := function( G )
    local n, t, gen, i, j;
    
    # check
    if not IsPcpGroup( G ) then return fail; fi;

    # get generators of G
    gen := GeneratorsOfGroup( G );
    n := Length( gen );
    
    # check orders of generators
    for i in [1..n] do
        if not IsInfinity(Order(gen[i])) then return fail; fi;
    od;
    
    # set up t
    t := [];
    for i in [1..n] do
        t[i] := 0*[1..n];
        for j in [1..n] do
            t[i][j] := 0*[1..n];
        od;
    od;
    
    # compute parameters
    for i in [1..n] do
        for j in [i+1..n] do
            t[i][j] := Exponents( Comm( gen[j], gen[i] ) );
            if not t[i][j]{[1..j]} = 0*[1..j] then return fail; fi;
        od;
    od;
    
    return t;
end;


#############################################################################
## 
## VectorByGroupCS( G )
## 
## Compute parameters t with G \cong G( t ), if possible. Uses the 
## upper central series.
## 
## Input: G = group
## 
VectorByGroupCS := function( G )
    local S, H;

    # check
    if not IsPcpGroup( G ) then return fail; fi;

    # get series
    S := UpperCentralSeries(G);
   
    # get new presentation
    H := PcpGroupBySeries(S, "snf");

    return VectorByGroupPcp(H);
end;

#############################################################################
## 
## HallRecByVector( t )
## 
## Returns HallRec for given parameters t.
##
HallRecByVector := function( t )
    local n, vars, imgs, i, j, k, HR;

    n := Length(t);
    if n > 7 then return fail; fi;

    vars := [];
    imgs := [];

    for i in [1..n] do
        for j in [i+1..n] do
            for k in [j+1..n] do
                Add(vars, T[i][j][k]);
                Add(imgs, t[i][j][k]);
            od;
        od;
    od;

    HR := List([1..n], x -> rec());
    for i in [1..n] do
        if i <= 2 then 
            HR[i].mult := 0;
            HR[i].powr := 0;
        else
            HR[i].mult := Value(HallRec[i].mult, vars, imgs);
            HR[i].cons := Set(List(HallRec[i].cons, 
                                x -> Value(x,vars,imgs)));
            HR[i].cons := Filtered(HR[i].cons, x -> x <> 0*x);
            HR[i].powr := Value(HallRec[i].powr, vars, imgs);
        fi;
    od;
    return HR;
end;

