#############################################################################
## 
## GenT( n )
## 
## Generate random parameters t for Hirsch-length n.
## If possible, check for consistency. ( results in many zeros )
## 
## Input: n = Hirsch-length
## 
GenT := function( n )
    local t, i, j, k, s, i1, j1, k1, s1, m, r, c, v, V, S, flag;
    
    t := [];
    
    # loops
    for i in [1..n] do
        t[i] := 0*[1..n];
        for j in [1..n] do
            t[i][j] := 0*[1..n];
        od;
    od;
    
    # generate random entries for t
    for i in [1..n] do
        for j in [i+1..n] do
            for k in [j+1..n] do
                t[i][j][k] := Random( [-100..100] );
            od;
        od;
    od;
    
    # if possible: check, whether t fulfills the consistency relations
    if Length( HallRec ) >= n then if Length( HallRec[n].cons ) > 0 then
        V := T{[1..n]}{[1..n]}{[1..n]};
        
        # get numbers of variables
        S := StructuralCopy( V );
        for i in [1..n] do
            for j in [1..n] do
                for k in [1..n] do
                    S[i][j][k] :=
                        IndeterminateNumberOfLaurentPolynomial( S[i][j][k] );
                od;
            od;
        od;
        S := Flat( S );
        
        # check each consistency polynomial and adjust t
        m := Length( HallRec[n].cons );
        for i in [1..m] do
            r := HallRec[n].cons[i];
            c := StructuralCopy( t );
            while Value( r, Flat( V ), Flat( t ) ) <> 0 do
                # get variables in r
                s := ExtRepPolynomialRatFun( r );
                s1 := [];
                for j in [1..Int( Floor( Float( Length( s )/2 ) ) )] do
                    s1[j] := s[2*j-1];  # number and exponent
                od;
                s1 := Flat( s1 );
                s := [];
                for j in [1..Int( Floor( Float( Length( s1 )/2 ) ) )] do
                    s[j] := s1[2*j-1];  # number
                od;
                s := Set( s );  # remove duplicates and sort
                
                # set some t[i][j][k] = 0 until r( t ) = 0
                while Value( r, Flat( V ), Flat( t ) ) <> 0 do
                    j := Random( s );
                    k := Position( S, j );
                    k1 := k mod n;
                    if k1=0 then k1 := n; fi;
                    j1 := ( k - k1 )/n mod n + 1;
                    i1 := ( k-k1-( j1-1 )*n )/( n*n ) + 1;
                    t[i1][j1][k1] := 0;
                od;
                
                # check, whether t fulfills the other consistency relations
                j := 1;
                flag := false;
                while j < i and flag = 0 do
                    s := HallRec[n].cons[j];
                    if Value( s, Flat( V ), Flat( t ) ) <> 0 then
                        flag := true;
                    fi;
                    j := j+1;
                od;
                
                # reset t if necessary
                if flag = true then
                    t := StructuralCopy( c );
                fi;
            od;
        od;
    fi; fi;
    
    return t;
end;


#############################################################################
## 
## GenT2( n, m )
## 
## Generate m random parameters t for Hirsch-length n.
## If possible, check for consistency. ( hopefully less zeros )
## 
## Input: n = Hirsch-length; m = quantity of solutions
## 
GenT2 := function( n, m )
    local i, j, k, l, s, r, t, iter, it, V, flag;
    
    t := [];
    
    # set up
    for l in [1..m] do
        t[l] := [];
        for i in [1..n] do
            t[l][i] := 0*[1..n];
            for j in [1..n] do
                t[l][i][j] := 0*[1..n];
            od;
        od;
    od;
    
    # generate random entries for t
    for l in [1..m] do
        for i in [1..n] do
            for j in [i+1..n] do
                for k in [j+1..n] do
                    #t[l][i][j][k] := Random( [-100..100] );
                    t[l][i][j][k] := 0;
                od;
            od;
        od;
    od;
    
    # if possible: check, whether t fulfills the consistency relations
    if Length( HallRec ) >= n then if Length( HallRec[n].cons ) > 0 then
        
        V := T{[1..n]}{[1..n]}{[1..n]};
        
        # set up iterator
        iter := [];
        #for i in [1..n*n*n] do
        #    iter[i] := [-100..100];
        #od;
        for i in [1..n] do
            for j in [1..i] do
                for k in [1..n] do
                    iter[( i-1 )*n*n+( j-1 )*n+k] := [0];
                od;
            od;
            for j in [i+1..n] do
                for k in [1..j] do
                    iter[( i-1 )*n*n+( j-1 )*n+k] := [0];
                od;
                for k in [j+1..n] do
                    iter[( i-1 )*n*n+( j-1 )*n+k] := [-100..100];
                od;
            od;
        od;
        
        l := Length( HallRec[n].cons );
        k := 1;
        s := 1;
        
        # iterate t until consistency relations are fulfilled
        for it in IteratorOfCartesianProduct( iter ) do
            flag := true;
            
            for j in [1..l] do
                r := HallRec[n].cons[j];
                
                if Value( r, Flat( V ), it ) <> 0 then
                    flag := false;
                    break;
                fi;
            od;
            
            if flag = true then
                if s = 11 or s = 12 or s = 13 then
                    Print( "- found ", String( s ), "th t\n" );
                elif s mod 10 = 1 then
                    Print( "- found ", String( s ), "st t\n" );
                elif s mod 10 = 2 then
                    Print( "- found ", String( s ), "nd t\n" );
                elif s mod 10 = 3 then
                    Print( "- found ", String( s ), "rd t\n" );
                else
                    Print( "- found ", String( s ), "th t\n" );
                fi;
                #t[s] := it;
                for i in [1..n] do
                    for j in [i+1..n] do
                        for k in [j+1..n] do
                            t[s][i][j][k] := it[( i-1 )*n*n+( j-1 )*n+k];
                        od;
                    od;
                od;
                if s = m then break; fi;
                s := s+1;
            fi;
            
            if k = 100000 then
                Print( "." );
                k := 0;
            fi;
            k := k+1;
        od;
        
    fi; fi;
    
    return t;
end;


#############################################################################
## 
## CheckHallMult( n )
## 
## Check computed Hall-polynomials for random parameters with random values.
## 
## Input: n = considered Hirsch-length
## 
CheckHallMult := function( n )
    local a, b, t, l, G, i, j, p, q, g, h;
    
    # setup
    a := 0*[1..n];
    b := 0*[1..n];
    
    for l in [1..100] do
        # random values for t ( with consistent G( t ) )
        t := GenT( n );
        
        # generate group
        G := GroupByVector( n, t );
        
        # generate random exponents & check if HallMult yields correct value
        for j in [1..100] do
            for i in [1..n] do
                a[i] := Random( -100, 100 );
    	        b[i] := Random( -100, 100 );
    	    od;
    	    
    	    # answer by using Hall-polynomials
    	    p := HallMult( t, a, b );
            
    	    # answer by using Collection
    	    g := MappedVector( a, GeneratorsOfGroup( G ) );
    	    h := MappedVector( b, GeneratorsOfGroup( G ) );
    	    q := Exponents( g*h );
    	    
    	    if p-q <> 0*( p-q ) then
    	        Print( "Error at \n t = ", t, " a = ", a, " b = ", b );
    	    fi;
    	od;
    	if l mod 10 = 0 then Print( l, ", " ); fi;
    od;
    
    return true;
end;


#############################################################################
## 
## CheckHallMultFixedT( n, t1, I )
## 
## Check computed Hall-polynomials for fixed parameters with random values.
## 
## Input: n = Hirsch-length; t1 = parameters t; I = index of Hall-polynomials
## 
CheckHallMultFixedT := function( n, t1, I )
    local t, a, b, G, i, j, p, q, g, h;
    
    if Length( t1 ) < n then
        Print( "check the input! \n" );
        return fail;
    fi;
    t := StructuralCopy( t1{[1..n]}{[1..n]}{[1..n]} );
    
    # generate group
    G := GroupByVector( n, t );
    
    # setup
    a := 0*[1..n];
    b := 0*[1..n];
    
    # generate random exponents & check if HallMult yields correct value
    for j in [1..1000] do
        for i in [1..n] do
            a[i] := Random( -100, 100 );
            b[i] := Random( -100, 100 );
        od;
        
        # answer by using Hall-polynomials
        p := HallMult( I, a, b );
        
        # answer by using Collection
        g := MappedVector( a, GeneratorsOfGroup( G ) );
        h := MappedVector( b, GeneratorsOfGroup( G ) );
	    q := Exponents( g*h );
	    
	    if p-q <> 0*( p-q ) then
	        Print( "Error at \n a = ", a, " b = ", b, "!\n j = ",
                                String( j ), "\n" );
            return false;
	    fi;
	od;
        
    return true;
end;

