#############################################################################
## 
## HallMult( t, a, b )
## 
## Let a and b be two vectors of length n.
## Let t be a parameter array
##
## The function determines F(t;a,b) as vector of length n.
## 
HallMult := function( t, a, b )
    local f, n, vars, imgs, i, j, k;
    
    # init
    if Length(a) <> Length(b) then return fail; fi;
    n := Length(a);
 
    # get images
    vars := [];
    imgs := [];
    for i in [1..n] do
        if a[i] <> A[i] then Add(vars, A[i]); Add(imgs, a[i]); fi;
        if b[i] <> B[i] then Add(vars, B[i]); Add(imgs, b[i]); fi;

        for j in [i+1..n] do
            for k in [j+1..n] do
                if t[i][j][k] <> T[i][j][k] then 
                    Add(vars, T[i][j][k]); Add(imgs, t[i][j][k]);
                fi;
            od;
        od;
    od;
    
    # determine multiplication
    f := a+b;
    for i in [3..n] do
        f[i] := f[i] + Value( HallRec[i].mult, vars, imgs );
    od;

    return f;
end;


#############################################################################
## 
## HallPower( t, a, x )
## 
## Let x be an indeterminate or an integer.
## Let a be a vector of length n.
## Let t = (t_{ijk} | 1 <= i < j < k <= n).
##
## The function determines K(t;a,x) as vector of length n.
##
HallPower := function( t, a, x )
    local f, n, vars, imgs, i, j, k, X;
    
    # init
    n := Length(a);
    X := Indeterminate(Rationals, "x");
 
    # get images
    vars := [];
    imgs := [];
    if x <> X then Add(vars, X); Add(imgs, x); fi;
    for i in [1..n] do
        if a[i] <> A[i] then Add(vars, A[i]); Add(imgs, a[i]); fi;
        for j in [i+1..n] do
            for k in [j+1..n] do
                if t[i][j][k] <> T[i][j][k] then 
                    Add(vars, T[i][j][k]); Add(imgs, t[i][j][k]);
                fi;
            od;
        od;
    od;
    
    # determine multiplication
    f := x*a;
    for i in [3..n] do
        f[i] := f[i] + Value( HallRec[i].powr, vars, imgs );
    od;

    return f;
end;


#############################################################################
## 
## HallMultByHR( HR, a, b)
## 
## Let a and b be two vectors of length n.
## Let HR be a Hall-record with entries .mult and .powr.
##
## The function determines F(a,b) as vector of length n.
## 
HallMultByHR := function( HR, a, b )
    local f, n, vars, imgs, i, j, k;
    
    # init
    if Length(a) <> Length(b) then return fail; fi;
    n := Length(a);
 
    # get images
    vars := [];
    imgs := [];
    for i in [1..n] do
        if a[i] <> A[i] then Add(vars, A[i]); Add(imgs, a[i]); fi;
        if b[i] <> B[i] then Add(vars, B[i]); Add(imgs, b[i]); fi;
    od;

    # determine multiplication
    f := a+b;
    for i in [3..n] do
        f[i] := f[i] + Value( HR[i].mult, vars, imgs );
    od;
    
    return f;
end;


#############################################################################
## 
## HallPowerHR( HR, a, x )
## 
## Let a be a vector of length n.
## Let HR be a Hall-record with entries .mult and .powr.
##
## The function determines K(a,x) as vector of length n.
##
HallPowerByHR := function( HR, a, x )
    local f, n, vars, imgs, i, j, k, X;
    
    # init
    n := Length(a);
    X := Indeterminate(Rationals, "x");
    
    # get images
    vars := [];
    imgs := [];
    if x <> X then Add(vars, X); Add(imgs, x); fi;
    for i in [1..n] do
        if a[i] <> A[i] then Add(vars, A[i]); Add(imgs, a[i]); fi;
    od;
    
    # determine multiplication
    f := x*a;
    for i in [3..n] do
        f[i] := f[i] + Value( HR[i].powr, vars, imgs );
    od;

    return f;
end;

