#############################################################################
## 
## CheckFileExists( n )
## 
## Check if the file for the polynomials of Hirsch-length n exists.
## 
## Input: Hirsch-length n
## 
CheckFileExists := function( n )
    local dir, file;
    
    dir := DirectoriesPackageLibrary( "hallpoly", "lib" )[1];
    file := Filename( dir, Concatenation( "hrl", String( n ), ".gi" ) );
        
    if IsExistingFile( file ) then
        return true;
    else
        return false;
    fi;
end;


#############################################################################
## 
## PrintPoly( file, f )
## 
## Print the polynomial f in the file file.
## 
## Input: file = path of file; f = polynomial;
##        flag = multiple lines (true) or one line (false)
## 
PrintPoly := function( file, f, flag )
    local e, i, j;
    
    e := ExtRepPolynomialRatFun( f );
    for i in [1, 3..Length( e )-1] do
 
        # print coeff
        if e[i+1] > 0 and i > 1 then 
           AppendTo( file, "+" );
        fi;
        if e[i+1] <> 1 then 
            AppendTo( file, e[i+1], "*" );
        fi;

        # print monomial
        for j in [1, 3..Length( e[i] )-1] do
            AppendTo( file, NAMES[e[i][j]] );
            if e[i][j+1] <> 1 then 
                AppendTo( file, "^", e[i][j+1] );
            fi;
            if j < Length( e[i] )-1 then 
                AppendTo( file, "*" );
            else
                if flag then
                    AppendTo( file, "\n" );
                else
                    AppendTo( file, " " );
                fi;
            fi;
        od;
    od;
end;


#############################################################################
## 
## SaveHallPolynomials( n )
## 
## Save the Hall-polynomials for Hirsch-length n.
## 
## Input: Hirsch-length n
## 
SaveHallPolynomials := function( n )
    local dir, file, path, i;
    
    if not IsBound(HallRec[n]) then
        Print("There are no Hall-polynomials to store.\n");
        return false;
    fi;
    
    if CheckFileExists(n) then
        Print("The file for the desired Hirsch-length already exists.\n");
        return false;
    fi;    
    
    dir := DirectoriesPackageLibrary( "hallpoly", "lib" )[1];
    file := Filename( dir, Concatenation( "hrl", String( n ), ".gi" ) );
    
    PrintTo( file, "\n" );
    
    # multiplication polynomial
    AppendTo( file, Concatenation( "HallRec[", String( n ), "].mult := " ) );
    PrintPoly( file, HallRec[n].mult, true );
    AppendTo( file, "; \n\n" );
    
    # power polynomial
    AppendTo( file, Concatenation( "HallRec[", String( n ), "].powr := " ) );
    PrintPoly( file, HallRec[n].powr, true );
    AppendTo( file, "; \n\n" );
    
    # consistency relations
    AppendTo( file, Concatenation( "HallRec[", String( n ), "].cons := " ) );
    AppendTo( file, "[ " );
    for i in [1..Length( HallRec[n].cons )] do
        PrintPoly( file, HallRec[n].cons[i], false );
        if i <> Length( HallRec[n].cons ) then
            AppendTo( file, ", \n  " );
        fi;
    od;
    AppendTo( file, "]; \n\n" );
    
    return true;
end;


#############################################################################
## 
## SaveParameters( t, file )
## 
## Save the parameters t. ( with a consistent presentation )
## 
## Input: t = parameters; file = filename
## 
SaveParameters := function( t, file )
    local dir, n, i, j, k;
    
    n := Length( t );
    dir := DirectoriesPackageLibrary( "hallpoly", "lib" )[1];
    file := Filename( dir, file );
    
    AppendTo( file, "\n\n" );
    
    AppendTo( file, "t := [" );
    for i in [1..n] do
        AppendTo( file, "[" );
        for j in [1..n] do
            AppendTo( file, "[" );
            for k in [1..n] do
                if k <> n then
                    AppendTo( file, Concatenation( 
                                String( t[i][j][k] ), ", " ) );
                else
                    AppendTo( file, String( t[i][j][k] ) );
                fi;
            od;
            if j <> n then
                AppendTo( file, "], " );
            else
                AppendTo( file, "]" );
            fi;
        od;
        if i <> n then
            AppendTo( file, "], \n" );
        else
            AppendTo( file, "]" );
        fi;
    od;
    AppendTo( file, "]\n\n" );

    return true;
end;

